/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.adapter;

/**
  * An <i>adapter delegate</i> determines how a value of
  * type {@code S} can be represented as type {@code T}. <p>
  * 
  * Adapter delegation normally imply a <i>strategy</i> of 
  * simple representation shift, and that the value represented 
  * as {@code E} and {@code T} are logically equivalent. This 
  * is not a requirement, however, and cannot be guaranteed. <p>
  * 
  * <b>Implementation notes</b>:<br>
  * This interface represents a <i>pluggable adapter</i> as described 
  * by Gamma et al. [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.144-145]. It is primarily used for 
  * sequence {@link SequenceAdapter adapters}, but can in theory adapt 
  * any two types. As a simple representation shift between types is 
  * normally implied, the {@linkplain dk.rode.thesis.strategy Strategy} 
  * pattern can be used to implement reusable adaptations.
  *  
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @see    SequenceAdapter
  * @see    AdapterStrategy
  * 
  * @param  <S> The adaptee type adapted by this adapter delegate.
  * @param  <T> The adapted (target) type delivered by this adapter delegate.
  */
public interface AdapterDelegate<S,T> {

    /**
      * Changes the representation of {@code value} from type {@code E}
      * to {@code T}.
      * 
      * @param  value The value; can be null, in which case null is 
      *               returned.
      * @return The value represented as type T; can be null.
      */
    public T adapt(S value);
    
}
