/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 * 
 */

/**
  * Implementations and examples of the <b>Bridge</b> 
  * design pattern [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.151]. <p>
  *  
  * <b>Intent</b>:<ul><p><i>
  * 
  *    Decouple an abstraction from its implementation so that the two 
  *    can vary independently.
  *     
  * </i></ul>
  *
  * Here, the <b>Abstraction</b> participant is represented by the 
  * {@link dk.rode.thesis.bridge.SequenceAbstraction} class,
  * which allows for abstraction regarding the 
  * {@link dk.rode.thesis.meta.model.Sequence} interface.  
  * The <b>RefinedAbstraction</b> participant is represented by
  * the {@link dk.rode.thesis.bridge.MemorizableSequenceAbstraction} 
  * class. <p>
  *
  * The <b>Implementor</b> participant is represented by the  
  * {@link dk.rode.thesis.bridge.SequenceValueGenerator} interface,
  * which defines the functionality needed for all implementors. A
  * refinement of this interface is the abstract class
  * {@link dk.rode.thesis.bridge.SequenceValueCollection}, which stores 
  * the actual sequence value delivered by a given sequence abstraction in
  * a collection. The actual {@code SequenceValueCollection} implementations defined
  * in this package thus represent the <b>ConcreteImplementor</b> participant,
  * for example the {@link dk.rode.thesis.bridge.SequenceValueArrayList}
  * and {@link dk.rode.thesis.bridge.SequenceValueTreeSet} classes. <p>
  * 
  * The {@link dk.rode.thesis.bridge.SequenceValueRange} class also 
  * corresponds to the <b>ConcreteImplementor</b> participant, but without
  * storing the values in a collection. <p>
  * 
  * The {@link dk.rode.thesis.bridge.SequenceValueGenerator#size() 
  * size}, {@link dk.rode.thesis.bridge.SequenceValueGenerator#ordered() 
  * ordered}, and {@link dk.rode.thesis.bridge.SequenceValueGenerator#duplicates() 
  * duplicate} properties of a {@code SequenceValueGenerator} makes it <b>very</b>
  * easy to alter the behaviour of a given sequence abstraction! <p>
  *  
  * <a name="uml"></a><b>UML Class Diagram</b>:<br>
  * <img src="doc-files/Bridge.png" border="0"><p>
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a> 
  */
@Pattern(name         = "Bridge", 
         scope        = Scope.Object,
         purpose      = Purpose.Structural,
         participants = {"Abstraction", "RefinedAbstraction", "Implementor", "ConcreteImplementor"}) 
package dk.rode.thesis.bridge;

import dk.rode.thesis.meta.Pattern;
import dk.rode.thesis.meta.Purpose;
import dk.rode.thesis.meta.Scope;






