/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.builder;

import dk.rode.thesis.interpreter.Expression;
import dk.rode.thesis.meta.Participant;
import dk.rode.thesis.meta.ParticipantUsage;

/**
  * A <i>comparable expression builder</i> construct various {@link Expression}
  * types having a {@code Comparable} bound on the value produced by 
  * the constructed expressions and/or on the expression operands used. <p>
  * 
  * The constructed <i>root expression</i> can be fetched using
  * the {@link #getRootExpression()} method, and expressions
  * can be added to it using the {@link #buildExpression(Expression)}
  * method. 
  *
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  *
  * @param  <E> The {@code Comparable} type of values the constructed expressions 
  *             operate on and/or produces.
  */
@ParticipantUsage(value = "Product", type = Expression.class)
@Participant("Builder")
public interface ComparableExpressionBuilder<E extends Comparable<? super E>> 
         extends ExpressionBuilder<E> {
    
    /**
      * Builds a new SMALLER THAN expression.
      * 
      * @param  first  The first expression operand; cannot be null.
      * @param  second The second expression operand; cannot be null.
      * @return The constructed expression; never null.
      * @throws NullPointerException If either expression is null.
      */
    public Expression<Boolean> buildSmallerThanExpression(Expression<E> first, Expression<E> second);
    
    /**
      * Builds a new GREATER THAN expression.
      * 
      * @param  first  The first expression operand; cannot be null.
      * @param  second The second expression operand; cannot be null.
      * @return The constructed expression; never null.
      * @throws NullPointerException If either expression is null.
      */
    public Expression<Boolean> buildGreaterThanExpression(Expression<E> first, Expression<E> second);

    /**
      * Builds a new SET expression.
      * 
      * @param  value The expression that will determine the sequence
      *               value to <i>fast-forward</i> to for the {@link #getSequence() 
      *               sequence} when the constructed expression is 
      *               evaluated.
      * @return The constructed expression; never null.                  
      * @throws NullPointerException If {@code value} is null.
      */
    public Expression<E> buildSetExpression(Expression<E> value);
 
}
