/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.composite;

import dk.rode.thesis.meta.Participant;
import dk.rode.thesis.meta.model.Sequence;

/**
  * A <i>char-sequence composite sequence</i> is a composite sequence 
  * parameterised with the {@code CharSequence} type. <p>
  * 
  * The value of {@link #current()} and {@link #next()} is the 
  * concatenation of the respective values returned by all 
  * associated sequences, in order.  
  *
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  */
@Participant("Composite")
public class CharSequenceCompositeSequence 
     extends AbstractCompositeSequence<CharSequence> 
  implements CompositeSequence<CharSequence> {

    /**
      * No-arg constructor.
      */
    public CharSequenceCompositeSequence() {
        super();
    }
    
    /**
      * Constructor.
      * 
      * @param strategy The traversal strategy; can be null, in which
      *                 case {@link CompositeStrategy#DEFAULT} is used. 
      */
    public CharSequenceCompositeSequence(CompositeStrategy strategy) {
        super(strategy);
    }

    /**
      * Copy constructor, which also copies associated sequences.
      * 
      * @param  sequence The composite sequence to copy; cannot be null.
      * @throws NullPointerException If {@code sequence} is null. 
      */
    public CharSequenceCompositeSequence(CompositeSequence<? extends CharSequence> sequence) {
        super(sequence);
    }

    /**
      * 
      */
    public CharSequenceCompositeSequence copy() {
        return new CharSequenceCompositeSequence(this);
    }

    /**
      * 
      */
    public CharSequence current() {
        StringBuilder sb = new StringBuilder().append("<[");
        for (int i = 0, n = this.sequences.size(); i < n; i++) {
            Sequence<? extends CharSequence> s = this.sequences.get(i);
            sb.append(s.current());
            if (i < n - 1) {
                sb.append("|");
            }
        }
        return sb.append("]>");
    }

    /**
      * 
      */
    public CharSequence next() {
        StringBuilder sb = new StringBuilder().append("<[");
        for (int i = 0, n = this.sequences.size(); i < n; i++) {
            Sequence<? extends CharSequence> s = this.sequences.get(i);
            sb.append(s.next());
            if (i < n - 1) {
                sb.append("|");
            }
        }
        // Calculate state...
        this.calculateNextState();
        
        return sb.append("]>");
    }
    
}
