/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.flyweight;

import dk.rode.thesis.meta.Participant;

/**
  * A <i>letter</i> represents a single letter that
  * is not a {@link Whitespace whitespace} or contain 
  * {@link Symbol symbolic} characters. <p>
  * 
  * A digit is not considered a letter, but a symbol.
  *
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  */
@Participant("ConcreteFlyweight")
public class Letter extends AbstractCharacter {

    /**
      * The actual shared string. <p>
      * 
      * For certain languages/alphabets, each letter might contain more
      * than a single character. <p>
      * 
      * Never null.
      */
    private final String letters;

    /**
      * Constructor.
      * 
      * @param letter The letter.
     */
    Letter(char letter) {
        this(java.lang.Character.toString(letter));
    }
    
    /**
      * Constructor.
      * 
      * @param  letters The letter(s) stored; cannot be null.
      * @throws NullPointerException If {@code letters} are null.
      */
    Letter(CharSequence letters) {
        super();
        this.letters = letters.toString();
    }
    
    /**
      * Returns true if this letter represents an upper case letter,
      * false if not.
      * 
      * @return True if upper case, false if not.
      */
    public final boolean isUpperCase() {
        for (char c : this.letters.toCharArray()) {
            if (java.lang.Character.isLowerCase(c)) {
                return false;
            }
        }
        return true;
    }
    
    /**
      * Returns true if this letter represents a lower case letter,
      * false if not.
      * 
      * @return True if lower case, false if not.
      */
    public final boolean isLowerCase() {
        for (char c : this.letters.toCharArray()) {
            if (java.lang.Character.isUpperCase(c)) {
                return false;
            }
        }
        return true;
    }
    
    /**
      * 
      */
    public final char charAt(int index) {
        return this.letters.charAt(index);
    }

    /**
      * 
      */
    public final int length() {
        return this.letters.length();
    }

    /**
      * 
      */
    public final CharSequence subSequence(int start, int end) {
        return this.letters.subSequence(start, end);
    }

    /**
      * 
      */
    public final String toString() {
        return this.letters;
    }
    
    /**
      * 
      */
    public final int compareTo(Character character) {
        return this.letters.compareTo(character.toString());
    }

}
