/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.iterator;

/**
  * A <i>value processor</i> is used by a {@link ProcessableSequence 
  * processable} sequence to process values during  
  * {@link ProcessableSequence#process(ValueProcessor) processing}.
  *
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @param  <E> The type of values processed by this processor.
  */
public interface ValueProcessor<E> {

    /**
      * Prepares this processor before {@link #process(Object) 
      * processing}. 
      */
    public void initialise();
    
    /**
      * Process a given value and returns false if the
      * processing is done, true if it must continue. 
      *  
      * @param  value The value to process; never null.
      * @return False to stop processing, true to continue.
      * @see    ProcessableSequence#process(ValueProcessor)
      */
    public boolean process(E value);
    
}
