/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.memento;

import static dk.rode.thesis.meta.util.Const.SERIAL_VERSION_UID;

import dk.rode.thesis.strategy.SequencePolicy;

/**
  * A <i>memorizable exception</i> is thrown in case a 
  * {@link MemorizableSequence memorizable} sequence cannot 
  * be updated by a given sequence {@link SequenceMemento memento}.
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  */
public class MemorizableException extends Exception {

    /**
      * Serial version id. 
      */
    final static long serialVersionUID = SERIAL_VERSION_UID;
    
    /**
      * No-arg constructor.
      */
    public MemorizableException() {
        super();
    }

    /**
      * Constructor.
      * 
      * @param  memento The faulting memento; cannot be null.
      * @throws NullPointerException If {@code memento} is null.
      */
    public MemorizableException(SequenceMemento<?> memento) {
        this(memento.toString(SequencePolicy.VERBOSE).toString());
    }

    /**
      * Constructor.
      * 
      * @param message The detail message; can be null.
      */
    public MemorizableException(String message) {
        super(message);
    }

    /**
      * Constructor.
      * 
      * @param cause The cause; can be null.
      */
    public MemorizableException(Throwable cause) {
        super(cause);
    }

}
