/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.memento;

import dk.rode.thesis.meta.Participant;
import dk.rode.thesis.meta.model.Sequence;

/**
  * A <i>memorizable sequence</i> can memorise its internal state
  * at a given time and have it restored again later by
  * using {@link SequenceMemento mementos}. 
  *
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @param  <E> The type of values delivered by this memorizable sequence.
  */
@Participant("Originator") 
public interface MemorizableSequence<E> extends Sequence<E> {

    /**
      * Returns a sequence memento representing the current state
      * of this sequence.
      * 
      * @return The memento; never null.
      */
    public SequenceMemento<E> getMemorizableState();
    
    /**
      * Updates this sequence to the state supplied as {@code memento}.
      * 
      * @param  memento The memento supplying the state; cannot be null.
      * @throws NullPointerException If {@code memento} is null.
      * @throws MemorizableException If the update fails, e.g. illegal
      *         memento state for this sequence, or illegal memento type.
      */
    public void setMemorizableState(SequenceMemento<E> memento) throws MemorizableException;
    
}
