/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 * 
 */

/**
  * Implementations and examples of the <b>State</b> 
  * design pattern [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.305]. <p>
  *  
  * <b>Intent</b>:<ul><p><i>
  * 
  *    Allow an object to alter its behaviour when its internal state 
  *    changes. The object will appear to change its class.
  *    
  * </i></ul>
  *
  * Two different State implementations have been made: 1) Standard
  * class/interface usage and 2) dynamic proxies. <p>
  * 
  * For case number 1), the <b>Context</b> participant is represented
  * by the {@link dk.rode.thesis.state.StateableSequence} 
  * interface, and more specifically by the implementing class  
  * {@link dk.rode.thesis.state.ReversiblePrimeSequence}. 
  * An abstract class implementing the basic functionality
  * for any stateable sequence is also defined, 
  * {@link dk.rode.thesis.state.AbstractStateableSequence},
  * which {@code ReversiblePrimeSequence} therefore inherits.<p>
  * 
  * The <b>State</b> participant is represented by the
  * {@link dk.rode.thesis.state.FunctionalState} interface
  * that describes how <b>ConcreteState</b> participants are
  * encapsulated and accessed. {@code ReversiblePrimeSequence} 
  * defines a private enumeration representing different 
  * <b>ConcreteState</b> implementations corresponding to 
  * sequence behaviour such as <i>initialise</i>, <i>next</i>, 
  * <i>reset</i>, etc. <p>
  *
  * For case number 2), the <b>Context</b> participant is represented
  * by the {@link dk.rode.thesis.state.StepSequence} type. {@code StepSequence} 
  * is a dynamic proxy that represents a sequence that will advance the sequence 
  * value a given step on each call to {@code next()}. Internally, it uses two 
  * states types, one representing even sequence values and one representing odd 
  * sequence values. The common sequence value type is the <b>State</b> participant,
  * while the even and odd implementations represent the <b>ConcreteState</b>
  * participant. Once the sequence value flips from even to odd or vice versa, the 
  * <i>target object</i> (even or odd instance) will change, not the sequence itself. 
  * This imitates dynamic inheritance as discussed by Gamma et al. [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.309]. <p>   
  * 
  * <a name="uml"></a><b>UML Class Diagram</b>:<br><br>
  * <img src="doc-files/State.png" border="0"><p>
  * 
  * <b>Implementation notes</b>:<br/>
  * State transitions are primarily initiated by the different
  * <b>ConcreteState</b> implementations, but also by the
  * {@code ReversiblePrimeSequence} upon construction and when 
  * {@link dk.rode.thesis.state.ReversiblePrimeSequence#reverse() 
  * reversed}. <p>
  * 
  * The {@linkplain dk.rode.thesis.memento Memento} implementation
  * illustrates how to guard access to public state methods in the
  * {@link dk.rode.thesis.memento.GuardedSequenceMemento} class.
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a> 
  */
@Pattern(name         = "State", 
         scope        = Scope.Object,
         purpose      = Purpose.Behavioural,
         participants = {"Context", "State", "ConcreteState"})
package dk.rode.thesis.state;

import dk.rode.thesis.meta.Pattern;
import dk.rode.thesis.meta.Purpose;
import dk.rode.thesis.meta.Scope;

